<?

 /*****************************************************************************
 * Filename: include/plugins/pager/backend.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."fs.php");
 require_once($_path["include"]."cms.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."form.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."history.php");
 require_once($_path["include"]."toolkit.data.php");

 require_once($_jasPath["project"]["include"]."backend/data.php");

 class Plugin_Backend_pager extends Base_Plugin_Backend
 {
  function main()
  {
   global $_pager_simple, $_config, $_jasPath;

   $this->plugin->l->read("jas");

   $this->tk=new ToolkitData();
   unset($this->tk->e);
   $this->tk->e=&$this->e;

   //create default config
   if(!$this->plugin->config("default"))
   {
    $d=Data::cache("pager_pluginConfigs", "add", "item");
    $d->setValue("key", "default");

    $this->e->clear();
    $this->cmsOutput=$this->cms($d);
    return true;
   }

   //pager simple mode, allows other parts (e.g. modules) to take this into account
   $_pager_simple=$this->plugin->config["simple"];

   //get main backend component
   if(!($main=$this->backend->component("main")))
   {
    $this->e->add("Unable to get main backend component", "error");
    return false;
   }

   //make backend render pager style sheet
   $main->backend->addStyleSheet($this->plugin->path["root"]."style/pager.css");


   //execute and render CMS
   if(request("cms"))
   {
    if(!($catnr=request("catnr"))||!is_numeric($catnr))
     $catnr=false;

    if(!($templatenr=request("templatenr"))||!is_numeric($templatenr))
     $templatenr=false;

    $d=Data::cache();

    $this->languageSetsHack["current"]=$this->plugin->l->sets; //cms badly overwrites some translations, store current sets
    $this->cmsOutput=$this->cms($d, ($d->action=="move"?Array("templatenr" => $templatenr):Array("catnr" => $catnr, "templatenr" => $templatenr)));
    $this->languageSetsHack["cms"]=$this->plugin->l->sets; //store cms sets
   }
   else
    $this->cmsOutput=false;

   //clear cache
   if($_config->userRoot())
    if(request("clearcache"))
     if(!$this->clearCache(request("clearcache")))
      return false;

   //modify quick categories list icons + url
   if($quick=$this->backend->component("backend/categories/quick")) //categories list found
   {
    $quick->path=false; //current page

    $d=Data::cache("pager_categories", "view", "all");
    $d->setShowFields("templatenr", "disabled", "symlink");

    if(!$d->get())
    {
     $this->e->add($d->e);
     return false;
    }

    while($d->next())
    {
     //determine page type
     if($d->v["disabled"])
      $type="page_disabled";
     else if($d->v["symlink"])
      $type="page_link";
     else if($d->v["templatenr"])
      $type="page";
     else
      $type=false;

     //modify icon
     if($type&&isset($quick->categories[$d->v["nr"]]))
      $quick->categories[$d->v["nr"]]["icon"]=$type;
    }

    if(isset($quick->categories[$this->plugin->config["rootpagenr"]]))
     $quick->categories[$this->plugin->config["rootpagenr"]]["icon"]="rootpage";
   }

   return true;
  }


  function output()
  {
   global $_config, $_path;

   //display cms output (rendered in main)
   if($this->cmsOutput)
    return $this->cmsOutput;
   else if(!$this->plugin->config("default")) //no default config, done
    return false;

   //display main
   if(!($catnr=request("catnr"))||!is_numeric($catnr))
    $catnr=false;

   if(!($templatenr=request("templatenr"))||!is_numeric($templatenr))
    $templatenr=false;

   if(!$catnr&&(!isset($this->plugin->config["rootpagenr"])||!($catnr=$this->plugin->config["rootpagenr"])))
    return false;

   if(isset($this->languageSetsHack))
    $this->plugin->l->sets=$this->languageSetsHack["current"]; //restore original sets

   $output="<div class=\"pager\">\n";

   $this->plugin->addVar("catnr", $catnr);

   $d=Data::cache("categories", "tree", "up");
   $d->moduleObject->extendBy("pager_categories");
   $d->setShowFields("name", "module", "templatenr", "disabled", "symlink");

   $d->clearValues();
   $d->setValue("catnr", $catnr);

   if(!$d->get()||!$d->search("nr", $catnr))
   {
    $this->e->add($d->e);
    return false;
   }
   else
   {
    //determine page type
    if($d->v["nr"]==$this->plugin->config["rootpagenr"])
     $type="rootpage_dim";
    else if($d->v["disabled"])
     $type="page_disabled";
    else if($d->v["symlink"])
     $type="page_link";
    else if($d->v["templatenr"])
     $type="page_dim";
    else
     $type="category_dim";


    //get crumb trail
    $d->reset();
    if($d->vmax>1) //skip first (root category)
     $d->next();

    $path=Array();
    while($d->next())
     $path[]=$this->tk->link($d->v["name"], Array(false, Array("catnr" => $d->v["nr"])));

    $output.="<div class=\"threequarter\">\n";

    //content
    $d->last();
    if($catnr==$this->plugin->config["rootpagenr"]||$d->v["templatenr"]&&!$d->v["symlink"]) //(root) page
     $output.=$this->layout($catnr, $templatenr); //show layout (may overwrite templatenr)
    else if($d->v["symlink"]) //symlink
    {
     $d->setAction("view");
     $d->setExtra("item");

     $d->clearValues();
     $d->setValue("nr", $catnr);

     $d->setShowFields("name", "urlname", "disabled", "symlink", "doctype", "javascript", "javascriptglobal", "stylesheet", "symlinkpage");

     $item=$this->tk->dataItem($d, Array("get" => 1));
     $item=preg_replace("!href=\".+data/categories/(\d+)/\"!", "href=\"".$_path["HTML"]["url"]."?catnr=\\1\"", $item);
     $output.=$item;
    }
    else //normal category
    {
     //get modules lists
     $r="";
     $modules=Array("categories");
     foreach($d->v["module"] as $module)
      if(!in_array($module, $modules))
       $modules[]=$module;

     foreach($modules as $module)
     {
      $tmp=Data::cache($module, "view", "list");

      if(!($field=$tmp->getRightsReference())) //no rights reference
       continue;

      $tmp->clearValues();
      $tmp->setValue($field, $catnr);

      if(!($list=$this->tk->dataList($tmp, Array("get" => 0))))
       continue;

      //link header/pager to modules page
      $list=preg_replace("!href=\"".$_path["HTML"]["url"]."(.*)\"!", "href=\"".$this->tk->url("modules/".$tmp->module."/")."\\1\"", $list);
      $list=preg_replace("!action=\"".$_path["HTML"]["url"]."(.*)\"!", "action=\"".$this->tk->url("modules/".$tmp->module."/")."\\1\"", $list);

      if($module=="categories") //show data list for category rather than just category
       $list=preg_replace("!href=\"([^\"]*)data/categories/(\d+)/\"!", "href=\"".$_path["HTML"]["url"]."?catnr=\\2\"", $list);

      $output.="<h2>".$this->tk->link($tmp->title, Array("modules/".$tmp->module."/", Array("catnr" => $catnr)))."</h2>\n";
      $output.=$list;
     }
    }

    $output.="</div>\n";

    $output.="<div class=\"quarter\">\n";

    $output.="<h3>".$this->tk->icon("page_dim", "small")." ".$this->plugin->l->t("Page")."</h3>\n";
    $output.=$this->page($catnr, $templatenr); //show template

    if($_config->userRoot())
    {
     $output.="<h3>".$this->tk->icon("execute_dim", "small")." ".$this->plugin->l->t("Actions")."</h3>\n";
     $output.=$this->actions($catnr, $templatenr); //show template

     $output.="<h3>".$this->tk->icon("cache_dim", "small")." ".$this->plugin->l->t("Cache")."</h3>\n";
     $output.=$this->cache($catnr, $templatenr); //show template
    }

    $output.="</div>\n";

    //title
    $this->backend->setTitle($this->tk->icon($type), $path);
   }

   if(isset($this->languageSetsHack))
    $this->plugin->l->sets=$this->languageSetsHack["cms"]; //restore cms sets

   $output.="</div>\n";
   return $output;
  }


  function actions($catnr, $templatenr)
  {
   if($this->plugin->config["simple"]) //simple mode
    $this->tk->menuItem("actions", $this->plugin->l->t("Advanced mode"), Array(false, Array("cms" => 1, "m" => "pager_pluginConfigs", "a" => "edit", "catnr" => $catnr, "templatenr" => $templatenr, "nr" => $this->plugin->config["nr"], "simple" => 0, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Switch to advanced mode"), "mode-advanced");
   else
    $this->tk->menuItem("actions", $this->plugin->l->t("Simple mode"), Array(false, Array("cms" => 1, "m" => "pager_pluginConfigs", "a" => "edit", "catnr" => $catnr, "templatenr" => $templatenr, "nr" => $this->plugin->config["nr"], "simple" => 1, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Switch to simple mode"), "mode-simple");

   $this->tk->menuItem("actions", $this->plugin->l->t("Pager config"), Array(false, Array("cms" => 1, "m" => "pager_pluginConfigs", "a" => "edit", "catnr" => $catnr, "templatenr" => $templatenr, "nr" => $this->plugin->config["nr"])), $this->plugin->l->t("Edit pager plugin configuration"), "edit");

   return $this->tk->menu("actions");
  }


  function cache($catnr, $templatenr)
  {
   $this->tk->menuItem("cache", $this->plugin->l->t("Clear pages"), Array(false, Array("catnr" => $catnr, "templatenr" => $templatenr, "clearcache" => "page")), $this->plugin->l->t("Clear page cache"), "cache_page");
   $this->tk->menuItem("cache", $this->plugin->l->t("Clear images"), Array(false, Array("catnr" => $catnr, "templatenr" => $templatenr, "clearcache" => "image")), $this->plugin->l->t("Clear image cache"), "cache_image");
   $this->tk->menuItem("cache", $this->plugin->l->t("Clear all"), Array(false, Array("catnr" => $catnr, "templatenr" => $templatenr, "clearcache" => "all")), $this->plugin->l->t("Clear all cache"), "cache_clear");

   return $this->tk->menu("cache");
  }


  function page($catnr, $templatenr)
  {
   global $_jasPath, $_config, $_session;

   //get page
   $d=Data::cache("categories", "view", "item");
   $d->moduleObject->extendBy("pager_categories");

   $d->setShowFields("name", "urlname", "templatenr", "disabled", "symlink");
   $d->setValue("nr", $catnr);

   if(!$d->get())
   {
    $this->e->add($d->e);
    return false;
   }

   if($d->next())
   {
    if($catnr==$this->plugin->config["rootpagenr"]||(!$d->v["templatenr"]&&!$d->v["symlink"]))
     $module="categories";
    else
     $module="pager_categories";

    //show link
    if($module=="categories"||!$d->v["disabled"])
    {
     if(isset($_SESSION["jses"]["default"])&&isset($_SESSION["jses"]["default"]["project"])&&$_SESSION["jses"]["default"]["project"]==$_session->get("project")) //default project
      $link=$_jasPath["HTML"]["root"];
     else //backend project
      $link=$_jasPath["HTML"]["current"].$_jasPath["projects"];

     if($catnr==$this->plugin->config["rootpagenr"]) //root page
      $this->tk->menuItem("page", $this->plugin->l->t("Show homepage"), $link, $this->plugin->l->t("Show page: ".$d->v["name"]), "view", Array("rel" => "external"));
     else //normal page
      $this->tk->menuItem("page", $this->plugin->l->t("Show page"), $link.($d->v["urlname"]?$d->v["urlname"]:$d->v["nr"])."/", $this->plugin->l->t("Show page: ".$d->v["name"]), "view", Array("rel" => "external"));
    }

    //cms options
    $this->tk->menuItem("page", $this->plugin->l->t("Add"), Array(false, Array("cms" => 1, "m" => "pager_categories", "a" => "add", "catnr" => $d->v["nr"], "templatenr" => ($templatenr?$templatenr:$d->v["templatenr"]), "moduleLock" => 0)), $this->plugin->l->t("Add a new page"), "add", ($d->r("add")?false:Array("disabled")));
    $this->tk->menuItem("page", $this->plugin->l->t("Edit"), Array(false, Array("cms" => 1, "m" => $module, "a" => "edit", "nr" => $d->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Edit page: ".$d->v["name"]), "edit", ($d->r("edit")?false:Array("disabled")));

    if(!$this->plugin->config["simple"])
    {
     $this->tk->menuItem("page", $this->plugin->l->t("Move"), Array(false, Array("cms" => 1, "m" => $module, "a" => "move", "nr" => $d->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Move page: ".$d->v["name"]), "move", ($d->r("move")?false:Array("disabled")));
     $this->tk->menuItem("page", $this->plugin->l->t("Delete"), Array(false, Array("cms" => 1, "m" => $module, "a" => "delete", "nr" => $d->v["nr"])), $this->plugin->l->t("Delete page: ".$d->v["name"]), "delete", ($d->r("delete")?false:Array("disabled")));
    }

    if(!$this->plugin->config["simple"])
    {
     $this->tk->menuItem("page", $this->plugin->l->t("Up"), Array(false, Array("cms" => 1, "m" => $module, "a" => "order", "e" => "up", "nr" => $d->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Order page: ".$d->v["name"]." up"), "up", ($d->r("edit")?false:Array("disabled")));
     $this->tk->menuItem("page", $this->plugin->l->t("Down"), Array(false, Array("cms" => 1, "m" => $module, "a" => "order", "e" => "down", "nr" => $d->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Order page: ".$d->v["name"]." down"), "down", ($d->r("edit")?false:Array("disabled")));
    }

    if($module=="pager_categories")
    {
     if($d->v["disabled"])
      $this->tk->menuItem("page", $this->plugin->l->t("Enable"), Array(false, Array("cms" => 1, "m" => $module, "a" => "edit", "nr" => $d->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "disabled" => 0, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Enable page: ".$d->v["name"]), "enable", ($d->r("edit")?false:Array("disabled")));
     else
      $this->tk->menuItem("page", $this->plugin->l->t("Disable"), Array(false, Array("cms" => 1, "m" => $module, "a" => "edit", "nr" => $d->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "disabled" => 1, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Disable page: ".$d->v["name"]), "disable", ($d->r("edit")?false:Array("disabled")));
    }
   }

   return $this->tk->menu("page");
  }


  function layout($catnr, &$templatenr)
  {
   global $_config, $_jses;

   $r="";

   $d=Data::cache("categories", "view", "item");
   $d->moduleObject->extendBy("pager_categories");

   $d->setShowFields("name", "urlname", "templatenr", "disabled");
   $d->setValue("nr", $catnr);
   $d->get(1);

   if($d->next())
   {
    if($d->v["nr"]==$this->plugin->config["rootpagenr"])
    {
     $isRoot=true;

     //get template
     $dt=Data::cache("pager_templates", "view", "all");
     $dt->setShowFields("name");

     if(!$dt->get())
     {
      $this->e->add($dt->e);
      return false;
     }

     if(!$templatenr)
      $templatenr=$dt->getValue("nr"); //use first template

     //cms options
     $this->tk->menuItem("template", $this->plugin->l->t("Add template"), Array(false, Array("cms" => 1, "m" => $dt->module, "a" => "add", "catnr" => $catnr)), $this->plugin->l->t("Add a new template"), "add", ($dt->r("add")?false:Array("disabled")));

     if($templatenr)
     {
      while($dt->next())
      {
       if($dt->v["nr"]!=$templatenr) //only show for current template
        continue;

       $this->tk->menuItem("template", $this->plugin->l->t("Edit template"), Array(false, Array("cms" => 1, "m" => $dt->module, "a" => "edit", "nr" => $dt->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Edit template: ".$dt->v["name"]), "edit", ($dt->r("edit")?false:Array("disabled")));

       if(!$this->plugin->config["simple"])
        $this->tk->menuItem("template", $this->plugin->l->t("Delete template"), Array(false, Array("cms" => 1, "m" => $dt->module, "a" => "delete", "nr" => $dt->v["nr"])), $this->plugin->l->t("Delete template: ".$dt->v["name"]), "delete", ($dt->r("delete")?false:Array("disabled")));
      }
     }

     $template=Array("");
     while($dt->next())
      $templates[$dt->v["nr"]]=$dt->v["name"];

     //template dropdown
     $r.=$this->tk->form("<div class=\"field\"><div class=\"title popup left\">".$this->tk->icon("template", "small").$this->tk->menu("template")." ".$this->plugin->l->t("Template")."</div><div class=\"input\">".$this->tk->f->select(false, "templatenr", $templates, $templatenr, false, false, "onchange=\"this.form.submit();\"")."<button type=\"submit\">".$this->tk->icon("view", "small")."</button></div></div>\n");
    }
    else if($d->getValue("templatenr"))
    {
     $isRoot=false;
     $templatenr=$d->getValue("templatenr");
    }
    else
    {




     return false;
    }

    //display template

    //get markers
    $dm=Data::cache("pager_markers", "view", "all");
    $dm->setValue("templatenr", $templatenr);
    $dm->get();

    //get content
    $dc=Data::cache("pager_content", "view", "all");
    $dc->setShowFields("nr", "catnr", "module", "contentnr", "markernr", "disabled", "displayfor", "cache", "symlink", "disabled");

    $dc->clearValues();
    $dc->setValue("catnr", $this->plugin->config["rootpagenr"]);
    $dc->setOperant("catnr", "OR");
    $dc->setValue("catnr", $catnr, 1);
    $dc->get();

    //get pluginConfigs list
    $tmp=Data::cache("pager_content", "add", "item"); //initializes pluginConfigs list
    $pluginConfigs=$tmp->listOptions["module"];
    unset($pluginConfigs[""]);
    unset($tmp);

    if($this->plugin->config["simple"]) //simple mode
    {
     $places=Array();

     //find active places
     foreach($dm->listOptions["placing"] as $place)
     {
      $dm->reset();
      while($dm->next())
      {
       if($dm->v["placing"]==$place)
       {
        $places[]=$place;
        break;
       }
      }
     }

     if(!$places) //no active places
     {
      //shown only 1 place
      if(sizeof($dm->listOptions["placing"])>1) //multiple places
       $places=Array($dm->listOptions["placing"][1]); //show 2nd
      else //1 place
       $places=Array($dm->listOptions["placing"][0]);
     }
    }
    else //show all places
     $places=$dm->listOptions["placing"];

    $r.="<div class=\"places\">";
    foreach($places as $place)
    {
     $r.="<table class=\"place ".$place.($d->v["disabled"]?" disabled":($isRoot?" root":""))."\"><tr>\n";
     $r.="<td class=\"name\">\n";
     $r.="<div class=\"place popup left\">\n";
     $r.=ucfirst($place);

     if($isRoot) //simple mode
     {
      $this->tk->menuItem($place, $this->plugin->l->t("Add marker"), Array(false, Array("cms" => 1, "m" => $dm->module, "a" => "add", "placing" => $place, "templatenr" => $templatenr, "catnr" => $this->plugin->config["rootpagenr"])), $this->plugin->l->t("Add a new marker"), "add", ($dm->r("add")?false:Array("disabled")));
      $r.=$this->tk->menu($place);
     }

     $r.="</div>\n";
     $r.="</td>\n";

     $columns=0;
     $dm->reset();
     while($dm->next())
      if($dm->v["placing"]==$place)
       $columns++;

     if($columns)
      $columnWidth=(1/$columns)*100; //width in %
     else
      $columnWidth=false;

     //show markers
     $dm->reset();
     while($dm->next())
     {
      if($dm->v["placing"]==$place)
      {
       //marker title and cms links
       $r.="<td class=\"values\" style=\"".($columnWidth&&$dm->v["small"]?"width: ".($columnWidth/2)."%;":"")."\">";

       if($this->plugin->config["simple"]) //simple mode
       {
        foreach($pluginConfigs as $plugin => $title)
         $this->tk->menuItem("marker_".$dm->v["nr"]."/plugins", $title, Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "add", "catnr" => $catnr, "templatenr" => $templatenr, "placing" => $place, "templatenr" => $templatenr, "marker" => html_entity_decode($dm->v["marker"], ENT_QUOTES|ENT_XHTML), "module" => $plugin)), $this->plugin->l->t("Add a new ".$title), "plugin", ($dc->r("add")?false:Array("disabled")));

        $this->tk->menuItem("marker_".$dm->v["nr"], $this->plugin->l->t("Add content"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "add", "catnr" => $catnr, "templatenr" => $templatenr, "placing" => $place, "templatenr" => $templatenr, "marker" => html_entity_decode($dm->v["marker"], ENT_QUOTES|ENT_XHTML))), $this->plugin->l->t("Add a new content"), "add", Array("menu" => "marker_".$dm->v["nr"]."/plugins", ($dc->r("add")?false:"disabled")));
       }
       else
       {
        foreach($pluginConfigs as $plugin => $title)
         $this->tk->menuItem("marker_".$dm->v["nr"]."/plugins", $title, Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "add", "catnr" => $catnr, "templatenr" => $templatenr, "markernr" => $dm->v["nr"], "module" => $plugin)), $this->plugin->l->t("Add a new ".$title), "plugin", ($dc->r("add")?false:Array("disabled")));

        $this->tk->menuItem("marker_".$dm->v["nr"], $this->plugin->l->t("Add content"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "add", "catnr" => $catnr, "templatenr" => $templatenr, "markernr" => $dm->v["nr"])), $this->plugin->l->t("Add a new content"), "add", Array("menu" => "marker_".$dm->v["nr"]."/plugins", ($dc->r("add")?false:"disabled")));
       }

       if($isRoot)
       {
        $this->tk->menuItem("marker_".$dm->v["nr"], $this->plugin->l->t("Edit marker"), Array(false, Array("cms" => 1, "m" => $dm->module, "a" => "edit", "nr" => $dm->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Edit marker: ".$dm->v["title"]), "edit", ($dm->r("edit")?false:Array("disabled")));

        if(!$this->plugin->config["simple"])
        {
         $this->tk->menuItem("marker_".$dm->v["nr"], $this->plugin->l->t("Delete marker"), Array(false, Array("cms" => 1, "m" => $dm->module, "a" => "delete", "nr" => $dm->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Delete marker: ".$dm->v["title"]), "delete", ($dm->r("delete")?false:Array("disabled")));

         if($columns>1)
         {
          $this->tk->menuItem("marker_".$dm->v["nr"], $this->plugin->l->t("Move marker left"), Array(false, Array("cms" => 1, "m" => $dm->module, "a" => "order", "e" => "up", "nr" => $dm->v["nr"], "_stp" => 1, "s" => 1, "templatenr" => $templatenr)), $this->plugin->l->t("Move marker: ".$dm->v["title"]." left"), "left", ($dm->r("edit")?false:Array("disabled")));
          $this->tk->menuItem("marker_".$dm->v["nr"], $this->plugin->l->t("Move marker right"), Array(false, Array("cms" => 1, "m" => $dm->module, "a" => "order", "e" => "down", "nr" => $dm->v["nr"], "_stp" => 1, "s" => 1, "templatenr" => $templatenr)), $this->plugin->l->t("Move marker: ".$dm->v["title"]." right"), "right", ($dm->r("edit")?false:Array("disabled")));
         }
        }
       }

       $r.="<div class=\"marker popup middle\">\n";
       $r.=$dm->v["title"].$this->tk->menu("marker_".$dm->v["nr"]);
       $r.="</div>\n";


       //show content for marker
       $c=0;
       while($dc->next())
       {
        if($dc->v["markernr"]==$dm->v["nr"])
        {
         //try to get content
         $dtmp=Data::cache($dc->v["module"], "view", "item");
         $dtmp->field["plugin"]["properties"]=str_replace("title", "", $dtmp->field["plugin"]["properties"]);
         $dtmp->setShowFields($dtmp->check("properties", "title"));

         $dtmp->setValue("nr", $dc->v["contentnr"]);
         $dtmp->get();
         $dtmp->next();

         //content menu
         if(($isRoot&&$dc->v["catnr"]==$this->plugin->config["rootpagenr"])||(!$isRoot&&$dc->v["catnr"]!=$this->plugin->config["rootpagenr"]))
         {
          $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Edit content"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "edit", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Edit content: ".$dtmp->v("title")), "edit", ($dc->r("edit")?false:Array("disabled")));

          if(!$this->plugin->config["simple"])
          {
           $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Move content"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "move", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Move content: ".$dtmp->v("title")), "move", ($dc->r("move")?false:Array("disabled")));
           $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Delete content"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "delete", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr)), $this->plugin->l->t("Delete content: ".$dtmp->v("title")), "delete", ($dc->r("delete")?false:Array("disabled")));

           $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Order content up"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "order", "e" => "up", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Order content: ".$dtmp->v("title")." up"), "up", ($dc->r("edit")?false:Array("disabled")));
           $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Order content down"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "order", "e" => "down", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Order content: ".$dtmp->v("title")." down"), "down", ($dc->r("edit")?false:Array("disabled")));
          }

          if($dc->v["disabled"])
           $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Enable"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "edit", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "disabled" => 0, "dontgetdata" => 1, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Enable content: ".$dtmp->v("title")), "enable", ($dc->r("edit")?false:Array("disabled")));
          else
           $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Disable"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "edit", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "disabled" => 1, "dontgetdata" => 1, "_stp" => 1, "s" => 1)), $this->plugin->l->t("Disable content: ".$dtmp->v("title")), "disable", ($dc->r("edit")?false:Array("disabled")));

          foreach(Array("Disabled", "Generic", "Anonymous", "User", "Group") as $cache)
          {
           $icon=($cache=="Generic"?"cache":($cache=="Anonymous"?"cache_anon":($cache=="Disabled"?"cache_disabled":("cache_".strtolower($cache)))));
           $this->tk->menuItem("content_".$dc->v["nr"]."/cache", $this->plugin->l->t($cache." cache"), Array(false, Array("cms" => 1, "m" => $dc->module, "a" => "edit", "nr" => $dc->v["nr"], "catnr" => $catnr, "templatenr" => $templatenr, "cache" => $cache, "dontgetdata" => 1, "_stp" => 1, "s" => 1)), $this->plugin->l->t($cache." cache for: ".$dtmp->v("title")), $icon, Array(($dc->r("edit")?false:"disabled"), ($dc->v["cache"]==$cache?"selected":false)));
          }

          if($dc->v["cache"]=="Disabled")
           $icon="cache_disabled";
          else if($dc->v["cache"]=="Anonymous")
           $icon="cache_anon";
          else if($dc->v["cache"]=="User")
           $icon="cache_user";
          else if($dc->v["cache"]=="Group")
           $icon="cache_group";
          else
           $icon="cache";

          $this->tk->menuItem("content_".$dc->v["nr"], $this->plugin->l->t("Cache"), false, false, $icon, Array("menu" => "content_".$dc->v["nr"]."/cache"));
         }

         //determine content type
         if($dc->v["catnr"]==$this->plugin->config["rootpagenr"])
         {
          if($dc->v["disabled"])
           $type="rootpage_disabled";
          else if($dc->v["symlink"])
           $type="rootpage_link";
          else
           $type="rootpage";
         }
         else if($dc->v["disabled"])
          $type="page_disabled";
         else if($dc->v["symlink"])
          $type="page_link";
         else
          $type="page";

         //render
         $r.="<div class=\"content popup middle\">\n";
         $r.=$this->tk->icon($type)." ".$dtmp->v("title");
         $r.=$this->tk->menu("content_".$dc->v["nr"]);
         $r.="</div>\n";
        }
       }

       $r.="</td>\n";
      }
     }

     $r.="</tr></table>\n";
    }
    $r.="</div>\n";
   }

   return $r;
  }


  function cms($d, $vars=false)
  {
   $r="";

   //set page as CMS in history
   $hs=new History();
   if(!$hs->cms())
    $this->e->add($hs->e);

   $cms=new CMS();
   foreach($this->plugin->vars as $k => $v)
    $cms->addVar($k, $v);

   if($vars&&is_array($vars))
    foreach($vars as $k => $v)
     $cms->addVar($k, $v);

   $cms->addVar("cms", "true");

   $d->dataformat="form";

   if($d->module=="pager_content"&&$d->action!="order")
   {
    if($d->action=="copy"||$d->action=="edit"||$d->action=="move"||$d->action=="delete") //get module
    {
     $d->get();

     if(request("dontgetdata")) //use submitted value
     {
      $d->setRequestValue("cache");
      $d->setRequestValue("markernr");
      $d->setRequestValue("disabled");
      $d->setRequestValue("displayfor");
      $d->setRequestValue("fixscript");
      $d->setRequestValue("fixstyle");
      $d->setRequestValue("fiximages");

      if($this->plugin->config["simple"]) //simple mode
      {
       $d->setRequestValue("templatenr");
       $d->setRequestValue("marker");
       $d->setRequestValue("placing");
      }

      if($d->getValue("symlink")) //use potential new contentnr for symlinks
       $d->setRequestValue("contentnr");
     }

     if($d->action=="move")
      $d->setRequestValue("catnr");
    }

    if(!$d->getValue("symlink")) //don't show content pick list for non symlinks
    {
     $d->field["contentnr"]["properties"].=";hidden";

     if($d->getValue("module"))
     {
      $dc=Data::cache($d->getValue("module"), false, false, Array("request" => "content", "format" => "form"));
      $dc->field["plugin"]["properties"].=";hidden";

      if($d->action=="add")
      {
       $d->field["catnr"]["properties"].=";hidden";

       if($dc->check("valuetype", "default", "catnr")) //use pager_content
        $dc->setValue("catnr", $d->getValue("catnr"));
       else //use module
        $d->setValue("catnr", $dc->getValue("catnr"));
      }
      else if($d->action=="move")
      {
       $refer=$dc->getRightsReference();
       $rightsReference=$dc->reference[$refer]["from"];

       $d->field["catnr"]["properties"].=";control";
       $d->setValue("catnr", $dc->getValue($rightsReference));
      }
     }
    }

    if($d->action=="edit"||$d->action=="move"||$d->action=="delete")
    {
     $d->field["module"]["properties"].=";disabled";
     $d->field["symlink"]["properties"].=";disabled";

     $d->unsetValue("ordernr");

     if(isset($dc))
      $dc->setValue("nr", $d->getValue("contentnr"));
    }
   }

   if(request("s"))
   {
    if(isset($dc)) //module+non symlink
    {
     $d->field["contentnr"]["min"]=0; //ensure d errorCheck doesn't fail on contentnr (set after dc is submitted)

     if(!$d->errorCheck(true)) //check d
     {
      $cms->e->add($d->e);
      $d->e->clear();

      if(!$dc->errorCheck(true)) //also get potential dc errors
      {
       $cms->e->add($dc->e);
       $dc->e->clear();
      }
     }
     else if(!$dc->submit()&&$dc->e->any("fatal")) //may not have any values set
     {
      $cms->e->add($dc->e);
      $dc->e->clear();
     }
     else
     {
      if($dc->action=="add")
      {
       $d->field["contentnr"]["min"]=1;
       $d->setValue("contentnr", $dc->getValue("nr"));
      }

      if(!$d->submit())
      {
       $cms->e->add($d->e);
       $d->e->clear();

       if($dc->action=="add") //clean up previously inserted
       {
        $dc->setAction("delete");
        $dc->submit();
        $dc->setAction("add");
       }
      }
      else
      {
       //$this->e->add($d->e);
       $this->e->add($dc->e);
       $d->e->clear();
       $dc->e->clear();

       if(!request("cmsContinue"))//oh you want more ?
        return false; //done
      }
     }
    }
    else //normal module / pager_content with symlink
    {
     if(!$d->submit())
     {
      $cms->e->add($d->e);
      $d->e->clear();
     }
     else
     {
      $this->e->add($d->e);
      $d->e->clear();

      if(!request("cmsContinue"))//oh you want more ?
       return false; //done
     }
    }
   }

   //handle reference manipulation
   if(isset($dc))
   {
    if($cms->referenceManipulation)
    {
     $cms->setData($d);
     $cms->formHandleReferenceManipulation();

     $cms->setData($dc);
     $cms->formHandleReferenceManipulation();
    }
   }
   else //normal module
   {
    if($cms->referenceManipulation)
    {
     $cms->setData($d);
     $cms->formHandleReferenceManipulation();
    }
   }

   $form=$cms->formStart(".");
   if(isset($dc))
   {
    $cms->setData($d);
    $form.=$cms->formShowFields();

    $cms->vars=Array(); //clear vars, already printed by previous formShowFields

    $cms->setData($dc);
    $form.=$cms->formShowFields();

    $cms->setData($d);

    $cms->e->add($d->e);
    $cms->e->add($dc->e);
   }
   else //normal module / pager_content with symlink
   {
    $cms->setData($d);
    $form.=$cms->formShowFields();

    $cms->e->add($d->e);
   }

   $form.=$cms->formShowSubmit();
   $form.=$cms->formShowCancel();
   $form.=$cms->formEnd();

   $r="<h2>".$d->title."</h2>\n";
   $r.=$cms->formShowErrors();
   $r.=$form;

   return $r;
  }


  function clearCache($type)
  {
   global $_path;

   $fs=new FS();

   switch($type)
   {
    case "page":
    {
     foreach($fs->files($_path["project"]["cache"]) as $file)
      if(preg_match("/\.html$/i", $file))
       $fs->delete($file);

     break;
    }
    case "image":
    {
     foreach($fs->files($_path["project"]["cache"]) as $file)
      if(preg_match("/\.(png|gif|jpg|jpeg)$/i", $file))
       $fs->delete($file);

     break;
    }
    case "all":
    {
     foreach($fs->files($_path["project"]["cache"]) as $file)
      $fs->delete($file);

     break;
    }
    default:
    {
     $this->e->add("Clear ".$type." cache not supported", "hack");
     return false;
    }
   }

   //errors during removal?
   if($fs->e->any("fatal"))
   {
    $this->e->add($fs->e);
    return false;
   }

   $this->e->add(ucfirst($type)." cache cleared", "note");
   return true;
  }

 }
?>
