<?

 /*****************************************************************************
 * Filename: include/plugins/pager/modules/categories.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_pager_categories extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->linkrels=Array("author", "contact", "help", "index", "license", "search", "first", "prev", "next", "last", "up");

   if($this->data->module!="categories")
    $this->extend("categories");

   $this->data->title="Pager categories";

   $this->data->field["text"]["title"]="Description (meta)";

   $this->data->addField("keywords", "regexp", "all", "", false, 255, "", "Keywords (meta)");
   $this->data->regexps("keywords", "/^([a-z0-9,\s]+)$/i");

   $this->data->addField("urlname", "regexp", "all", "", false, 25, "", "URL name");
   $this->data->regexps("urlname", "/^([a-z0-9_]+)$/i");
   $this->data->addUnique("urlname", "urlname");

   $this->data->addField("disabled", "boolean", "all", "", false, false, false, "Disabled");
   $this->data->addField("symlink", "boolean", "all", "reload", false, false, false, "Link", "Link to an existing page");

   $this->data->addField("pagenr", "nr", "all", "hidden", 0, false, 0, "Page");
   $this->data->addReference("symlinkpage", "categories", "nr", "*:0", "pagenr");
   $this->data->field["symlinkpage"]["title"]="Linked page";

   $this->data->addField("templatenr", "nr", "all", "", 1, false, 0, "Template");
   $this->data->addReference("templatenr", "pager_templates", "nr", "*:1");


   $this->data->addField("doctype", "enum", "all", false, false, false, "Use template", "Alternative Doctype");
   $this->data->enumOptions("doctype", "Use template", "XHTML 1.1 - Strict", "XHTML 1.0 - Strict", "XHTML 1.0 - Transitional", "XHTML 1.0 - Frameset", "HTML 4.01 - Strict", "HTML 4.01 - Transitional", "HTML 4.01 - Quirks mode", "HTML 4.01 - Frameset");

   $this->data->addField("javascriptglobal", "localfiles", "all", "recursive", false, false, false, "Additional global JavaScript");
   $this->data->addField("javascript", "localfiles", "all", "recursive", false, false, "", "Additional JavaScript");
   $this->data->addField("stylesheet", "localfiles", "all", "recursive", false, false, "", "Additional Style sheets");

   $linkrelGroup=Array();
   foreach($this->linkrels as $rel)
   {
    if($rel=="first")
    {
     $this->data->addField("labellink", "label", "form", false, false, false, false, "");
     $linkrelGroup[]="labellink";
    }

    $this->data->addField("link".$rel, "text", "all", "", 0, 255, false, ucfirst($rel));
    $linkrelGroup[]="link".$rel;
   }

   $this->data->group("Template", "doctype", "javascriptglobal", "javascript", "stylesheet");
   $this->data->group("Category", "ordernr", "moduleLock", "module");
   $this->data->group("Link relations", $linkrelGroup);

   $this->data->sortOrder("ordernr", "title");
  }


  function viewPostGet()
  {
   for($c=0;$c<$this->data->vmax;$c++)
    if(!$this->data->getValue("urlname", $c))
     $this->data->setValue("urlname", $this->data->getValue("nr", $c), $c);

   if(!$this->customizeFields())
    return false;

   return true;
  }

  function postInit()
  {
   global $_path, $_PAGER_CATEGORIES_INIT;

   if($_PAGER_CATEGORIES_INIT)
    return true;

   $_PAGER_CATEGORIES_INIT=true;

   $this->data->setFileDir("javascriptglobal", $_path["script"]);
   $this->data->setFileExtensions("javascriptglobal", "js");
   $this->data->setFileDir("javascript", $_path["project"]["script"]);
   $this->data->setFileExtensions("javascript", "js");
   $this->data->setFileDir("stylesheet", $_path["project"]["style"]);
   $this->data->setFileExtensions("stylesheet", "css");

   if(!$this->simpleFields())
   {
    $_PAGER_CATEGORIES_INIT=false;
    return false;
   }

   if(!$this->customizeFields())
   {
    $_PAGER_CATEGORIES_INIT=false;
    return false;
   }

   $pages=Plugin_Module_pager_categories::getPages();
   foreach($this->linkrels as $rel)
    $this->data->listOptions("link".$rel, $pages);

   $_PAGER_CATEGORIES_INIT=false;
   return true;
  }

  function copyPostGet(){return $this->customizeFields();}
  function editPostGet(){return $this->customizeFields();}
  function movePostGet(){return $this->customizeFields();}
  function deletePostGet(){return $this->customizeFields();}

  function simpleFields()
  {
   global $_pager_simple;

   //hide "advanced" fields in simple mode
   if($_pager_simple)
   {
    $this->data->field["catnr"]["properties"].=";hidden";
    $this->data->field["ordernr"]["properties"].=";hidden";
    $this->data->field["module"]["properties"].=";hidden";
    $this->data->field["moduleLock"]["properties"].=";hidden";

    $this->data->field["disabled"]["properties"].=";hidden";
    $this->data->field["symlink"]["properties"].=";hidden";

    $this->data->field["doctype"]["properties"].=";hidden";

    $this->data->field["javascriptglobal"]["properties"].=";hidden";
    $this->data->field["javascript"]["properties"].=";hidden";
    $this->data->field["stylesheet"]["properties"].=";hidden";

    $this->data->field["labellink"]["properties"].=";hidden";
    foreach($this->linkrels as $rel)
     $this->data->field["link".$rel]["properties"].=";hidden";
   }
   else
   {
    $this->data->resetField("catnr");
    $this->data->resetField("ordernr");
    $this->data->resetField("module");
    $this->data->resetField("moduleLock");

    $this->data->resetField("disabled");
    $this->data->resetField("symlink");

    $this->data->resetField("doctype");

    $this->data->resetField("javascriptglobal");
    $this->data->resetField("javascript");
    $this->data->resetField("stylesheet");

    $this->data->resetField("labellink");
    foreach($this->linkrels as $rel)
     $this->data->resetField("link".$rel);
   }

   return true;
  }


  function customizeFields()
  {
   if($this->data->getValue("symlink"))
   {
    $this->data->field["pagenr"]["min"]=1;
    $this->data->field["symlinkpage"]["properties"]=str_replace(";hidden", "", $this->data->field["symlinkpage"]["properties"]); //remove hidden property, resetField does to much in this case
    $this->data->reference["symlinkpage"]["type"]="*:1";

    $this->data->field["templatenr"]["min"]=0;
    $this->data->reference["templatenr"]["type"]="*:0";
    $this->data->field["templatenr"]["properties"].=";hidden";

    $this->data->field["doctype"]["properties"].=";hidden";
    $this->data->field["javascriptglobal"]["properties"].=";hidden";
    $this->data->field["javascript"]["properties"].=";hidden";
    $this->data->field["stylesheet"]["properties"].=";hidden";
   }
   else
   {
    $this->data->field["pagenr"]["min"]=0;
    $this->data->reference["symlinkpage"]["type"]="*:0";
    $this->data->field["symlinkpage"]["properties"].=";hidden";

    $this->data->reference["templatenr"]["type"]="*:1";
    $this->data->resetField("templatenr");
   }

   return true;
  }


  function addPreSubmitErrorCheck()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if(!$this->data->getValue("urlname", $c))
    {
     $name=str_replace(" ", "_", $this->data->getValue("name", $c)); //replace spaces
     $name=preg_replace("/[^a-z0-9\_]/i", "", $name);
     $name=strtolower($name);
     $this->data->setValue("urlname", $name, $c);
    }
   }

   return $this->checkSymlink();;
  }

  function editPreSubmitErrorCheck(){return $this->checkSymlink();}

  function checkSymlink()
  {
   for($c=0;$c<$this->data->vmax;$c++)
   {
    if($this->data->getValue("symlink", $c))
    {
     if($this->data->getValue("pagenr", $c)==$this->data->getValue("nr", $c))
     {
      $this->data->e->add("Can't link to current page", "error");
      return false;
     }

     if(!isset($d))
     {
      $d=Data::cache("categories", "view", "item");
      $d->moduleObject->extendBy("pager_categories");
     }

     $d->clearValues();
     $d->setValue("nr", $this->data->getValue("pagenr", $c));

     if(!$d->get()||!$d->next())
     {
      $this->data->e->add("Linked page doesn't exist", "error");
      return false;
     }

     if($d->v["symlink"])
     {
      $this->data->e->add("Can't link to a linked page", "error");
      return false;
     }

     if(!$d->v["templatenr"])
     {
      $this->data->e->add("Linked page isn't a page", "error");
      return false;
     }
    }
   }

   return true;
  }

  static function getPages()
  {
   $d=Data::cache("pager_categories", "view", "all");
   $d->setShowFields("urlname", "name");
   $d->sortOrder("name");

   if(!$d->get())
    return false;

   $pages=Array("" => "", "/" => "Root");
   while($d->next())
    $pages["/".$d->v["urlname"]."/"]=$d->v["name"];

   return $pages;
  }
 }
?>
