<?

 /*****************************************************************************
 * Filename: include/plugins/pager/modules/content.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");

 class Plugin_Module_pager_content extends Module
 {
  function __construct(&$data)
  {
   global $_config;

   parent::__construct($data);

   $this->data->title="Page content";

   $this->data->addField("ordernr", "nr", "sql", "order", false, false, 0, "Order number", "Leave: 0 for automatic number");
   $this->data->addField("cache", "enum", "all", "", false, false, "Disabled", "Cache method", "Cache can be generic for all users, specific for anonymous only, specific per (main) group, specific per user of disabled. Note that file based caching is required");
   $this->data->enumOptions("cache", "Disabled", "Generic", "Anonymous", "Group", "User");

   $this->data->addField("markernr", "nr", "all", "", 1, false, 0, "Marker");
   $this->data->addReference("markernr", "pager_markers", "nr", "*:1");


   $this->data->addField("disabled", "boolean", "all", "", false, false, false, "Disabled");
   $this->data->addField("displayfor", "enum", "all", "", 0, false, "All", "Display for");
   $this->data->enumOptions("displayfor", "All", "Anonymous", "Logged in", "Root");


   $this->data->addField("module", "list", "all", "reload;title", 1, 255, false, "Plugin");
   $this->data->listOptions("module", Array("" => ""));

   $this->data->addField("symlink", "boolean", "all", "reload", false, false, false, "Link", "Link to an existing plugin configuration");

   $this->data->addField("contentnr", "nr", "all", "", 1, false, 0, "Content");


   $this->data->addField("fixscript", "boolean", "all", false, false, false, false, "Fix JavaScript paths");
   $this->data->addField("fixstyle", "boolean", "all", false, false, false, false, "Fix Style sheet paths");
   $this->data->addField("fiximages", "boolean", "all", false, false, false, false, "Fix image paths");


   //fake field for simple mode
   $this->data->addField("templatenr", "nr", "form;show", "reload", 1, false, 0, "Template");
   $this->data->addReference("templatenr", "pager_templates", "nr", "*:1");

   $this->data->addField("marker", "list", "form;show", "", 1, 255, false, "Marker");
   $this->data->addField("placing", "text", "form;show", "hidden", false, false, false, "Marker place");


   $this->data->defaultFields();


//   $this->data->addReference("pagerContentDeniedUsers", $_config->get("usermodule"), "nr", "*:*", "nr");
//   $this->data->field["pagerContentDeniedUsers"]["title"]="Deny users";


   $this->data->group("Display", "disabled", "displayfor");
   $this->data->group("Path fixing", "fixscript", "fixstyle", "fiximages");
   $this->data->group("Template", "templatenr", "marker", "placing");

   $this->data->sortOrder("catnr", "markernr", "ordernr");

   $this->referenceInitted=false;
  }


  function postInit()
  {
   if($this->data->action=="add"||$this->data->action=="copy"||$this->data->action=="edit")
   {
    $plugins=Array("" => "");
    foreach(JAS::plugins() as $p)
    {
     if($p!="pager")
     {
      $tmp=new Plugin($p);

      if($tmp->isEnabled())
       if(in_array($p."_pluginConfigs", JAS::modules("plugins", $p)))
        if($tmp->classExists("frontend")&&$tmp->info())
         $plugins[$p."_pluginConfigs"]=$tmp->title;
     }
    }

    asort($plugins);

    $this->data->listOptions("module", $plugins);
   }

   if(!$this->initReference())
    return false;

   if(!$this->simpleFields())
    return false;

   if(!$this->customizeFields())
    return false;

   return true;
  }


  /*
   * Get functions
   */

  function copyPostGet()
  {
   if(!$this->initReference())
    return false;

   if(!$this->customizeFields())
    return false;

   return true;
  }
  function editPostGet()
  {
   if(!$this->initReference())
    return false;

   if(!$this->customizeFields())
    return false;

   return true;
  }
  function movePostGet()
  {
   if(!$this->initReference())
    return false;

   if(!$this->customizeFields())
    return false;

   return true;
  }
  function deletePostGet()
  {
   if(!$this->initReference())
    return false;

   if(!$this->customizeFields())
    return false;

   return true;
  }


  /*
   * Submit functions
   */

  function addPreSubmitErrorCheck(){return $this->setMarker();}
  function copyPreSubmitErrorCheck(){return $this->setMarker();}
  function editPreSubmitErrorCheck(){return $this->setMarker();}

  function editPreSubmit() //make sure module+symlink fields aren't saved
  {
   $this->data->field["module"]["valuetype"]="none";
   $this->data->field["symlink"]["valuetype"]="none";
   return true;
  }


  /*
   * Helper functions
   */

  function initReference()
  {
   global $_pager_simple;

   if($this->referenceInitted)
    return true;

   if(($module=$this->data->getValue("module"))&&!is_array($module))
   {
    if(!$_pager_simple)
    {
     $this->data->resetField("cache");
     $this->data->resetField("symlink");
    }

    $this->data->resetField("contentnr");

    $plugin=str_replace("_pluginConfigs", "", $module);
    $plugin=new Plugin($plugin);

    if($plugin->e->any("fatal"))
    {
     $this->data->e->add($plugin->e);
     return false;
    }
    if(!$plugin->isEnabled())
    {
     $this->data->e->add("Plugin: \"".$plugin->name."\" is disabled", "error");
     return false;
    }

    $tmp=Data::cache($module, "view", "item");
    if($tmp->e->any("fatal"))
    {
     $this->data->e->add($tmp->e);
     return false;
    }

    $this->data->addReference("contentnr", $tmp->module, $tmp->getID(), "*:1");
    $this->referenceInitted=true;
   }
   else
   {
    $this->data->field["cache"]["properties"].=";hidden";
    $this->data->field["symlink"]["properties"].=";hidden";
    $this->data->field["contentnr"]["properties"].=";hidden";
   }

   return true;
  }

  function simpleFields()
  {
   global $_pager_simple;

   //hide "advanced" fields and show fake fields in simple mode
   if($_pager_simple)
   {
    $this->data->field["catnr"]["properties"].=";hidden";
    $this->data->field["ordernr"]["properties"].=";hidden";
    $this->data->field["cache"]["properties"].=";hidden";

    $this->data->field["markernr"]["min"]=0;
    $this->data->reference["markernr"]["type"]="*:0";
    $this->data->field["markernr"]["properties"].=";hidden";

    $this->data->field["disabled"]["properties"].=";hidden";
    $this->data->field["displayfor"]["properties"].=";hidden";
    $this->data->field["symlink"]["properties"].=";hidden";

    $this->data->field["fixscript"]["properties"].=";hidden";
    $this->data->field["fixstyle"]["properties"].=";hidden";
    $this->data->field["fiximages"]["properties"].=";hidden";


    $this->data->resetField("templatenr");
    $this->data->reference["templatenr"]["type"]="*:1";

    $this->data->resetField("marker");
    $this->data->resetField("placing");
   }
   else
   {
    $this->data->resetField("catnr");
    $this->data->resetField("ordernr");
    $this->data->resetField("cache");

    $this->data->resetField("markernr");
    $this->data->reference["markernr"]["type"]="*:1";

    $this->data->resetField("disabled");
    $this->data->resetField("displayfor");
    $this->data->resetField("symlink");

    $this->data->resetField("fixscript");
    $this->data->resetField("fixstyle");
    $this->data->resetField("fiximages");


    $this->data->field["templatenr"]["min"]=0;
    $this->data->reference["templatenr"]["type"]="*:0";
    $this->data->field["templatenr"]["properties"].=";hidden";

    $this->data->field["marker"]["min"]=0;
    $this->data->field["marker"]["properties"].=";hidden";
    $this->data->field["placing"]["properties"].=";hidden";
   }

   return true;
  }


  function customizeFields()
  {
   global $_pager_simple;

   if($_pager_simple)
   {
    //unset required fields in order mode (should be done by Data)
    if($this->data->action=="order")
    {
     $this->data->unsetValue("templatenr");
     $this->data->unsetValue("marker");
    }

    //marker set, get simple template+marker+placing
    if($this->data->getValue("markernr"))
    {
     $dm=Data::cache("pager_markers", $this->data->action, "item");
     $dm->setShowFields("templatenr", "marker", "placing");
     $dm->setValue("nr", $this->data->getValue("markernr"));

     if(!$dm->get())
     {
      $this->data->e->add($dm->e);
      return false;
     }

     $this->data->setValue("templatenr", $dm->getValue("templatenr"));
     $this->data->setValue("marker", html_entity_decode($dm->getValue("marker"), ENT_QUOTES|ENT_XHTML));
     $this->data->setValue("placing", $dm->getValue("placing"));
    }

    //templatenr set. get markers list
    if($this->data->getValue("templatenr"))
    {
     //get marker list from pager_markers module
     if(!isset($dm))
      $dm=Data::cache("pager_markers");

     $dm->setValue("templatenr", $this->data->getValue("templatenr"));
     $dm->moduleSpecificFunction("customizeFields", false);

     $this->data->listOptions("marker", $dm->listOptions["marker"]);
    }
   }

   return true;
  }


  function setMarker()
  {
   global $_pager_simple;

   //skip for normal mode
   if(!$_pager_simple)
    return true;

   if($this->data->extra=="up"||$this->data->extra=="down") //order up/down
    return true; //skip setMarker

   $pager=new Plugin("pager");

   if(!$pager->config("default"))
   {
    $this->data->e->add($pager->e);
    return false;
   }

   $dm=Data::cache("pager_markers", "view", "all");
   $dm->setShowFields("nr");

   for($c=0;$c<$this->data->vmax;$c++)
   {
    //check marker
    $dm->clearValues();
    $dm->e->clear();

    $dm->setValue("marker", $this->data->getValue("marker", $c));
    $dm->setValue("templatenr", $this->data->getValue("templatenr", $c));
    $dm->setValue("catnr", $pager->config["rootpagenr"]);

    if(!$dm->get())
    {
     $this->data->e->add($dm->e);
     return false;
    }

    if(!$dm->next()) //marker not found
    {
     //create new marker
     $dm->setAction("add");
     $dm->setExtra("item");
     $dm->clearValues();
     $dm->e->clear();

     $dm->setValue("marker", $this->data->getValue("marker", $c));
     $dm->setValue("title", $this->data->listOptions["marker"][$this->data->getValue("marker", $c)]);
     $dm->setValue("templatenr", $this->data->getValue("templatenr", $c));
     $dm->setValue("placing", $this->data->getValue("placing", $c));
     $dm->setValue("catnr", $pager->config["rootpagenr"]);
     $dm->setValue("clean", true); //clean marker before adding content

     if(!$dm->submit())
     {
      $this->data->e->add($dm->e);
      return false;
     }

     $dm->setAction("view");
     $dm->setExtra("all");
    }

    $this->data->setValue("markernr", $dm->getValue("nr"), $c);
   }

   return true;
  }

 }
?>
