<?

 /*****************************************************************************
 * Filename: include/plugins/pager/modules/templates.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."module.php");
 require_once(dirname(__FILE__)."/categories.php"); //use current plugin's module dir (could be system or project plugin)

 class Plugin_Module_pager_templates extends Module
 {
  function __construct(&$data)
  {
   parent::__construct($data);

   $this->linkrels=Array("author", "contact", "help", "index", "license", "search");

   $this->data->title="Page templates";

   $this->data->defaultFields();

   $this->data->addField("name", "text", "all", "title", 1, 50, "Default", "Template name");
   $this->data->addField("title", "text", "all", "", 0, 50, false, "Website title");

   $this->data->addField("keywords", "regexp", "all", "", false, 255, "", "Default keywords (meta)");
   $this->data->regexps("keywords", "/^([a-z0-9,\s]+)$/i");

   $this->data->addField("template", "localfile", "all", "recursive;reload", 1, 255, "index.html", "Template");

   $this->data->addField("markers", "multilist", "form", "disabled", false, false, false, "Markers in template");
   $this->data->enumOptions("markers", "STYLESHEET", "JAVASCRIPT", "LASTUPDATE", "GENERATED");

   $this->data->addField("doctype", "enum", "all", false, false, false, "XHTML 1.1 - Strict", "Doctype");
   $this->data->enumOptions("doctype", "XHTML 1.1 - Strict", "XHTML 1.0 - Strict", "XHTML 1.0 - Transitional", "XHTML 1.0 - Frameset", "HTML 4.01 - Strict", "HTML 4.01 - Transitional", "HTML 4.01 - Quirks mode", "HTML 4.01 - Frameset");

   $this->data->addField("jasscript", "boolean", "all", false, false, false, true, "Add JAS JavaScript");
   $this->data->addField("javascriptglobal", "localfiles", "all", "recursive", false, false, false, "Global JavaScript");
   $this->data->addField("javascript", "localfiles", "all", "recursive", false, false, false, "JavaScript");
   $this->data->addField("stylesheet", "localfiles", "all", "recursive", false, false, false, "Style sheets");

   $this->data->addField("fixscript", "boolean", "all", false, false, false, true, "Fix JavaScript paths");
   $this->data->addField("fixstyle", "boolean", "all", false, false, false, true, "Fix Style sheet paths");
   $this->data->addField("fiximages", "boolean", "all", false, false, false, true, "Fix image paths");

   $linkrelGroup=Array();
   foreach($this->linkrels as $rel)
   {
    $this->data->addField("link".$rel, "text", "all", "", 0, 255, false, ucfirst($rel));
    $linkrelGroup[]="link".$rel;
   }

   $this->data->group("JavaScript", "javascriptglobal", "javascript", "jasscript");
   $this->data->group("Style sheet", "stylesheet");
   $this->data->group("Path fixing", "fixscript", "fixstyle", "fiximages");
   $this->data->group("Link relations", $linkrelGroup);

   $this->data->sortOrder("title");
  }

  function postInit()
  {
   global $_path, $_PAGER_TEMPLATES_INIT;

   if($_PAGER_TEMPLATES_INIT)
    return true;

   $_PAGER_TEMPLATES_INIT=true;

   $this->data->setFileDir("template", $_path["project"]["templates"]);
   $this->data->setFileExtensions("template", "html", "htm");
   $this->data->setFileDir("javascriptglobal", $_path["script"]);
   $this->data->setFileExtensions("javascriptglobal", "js");
   $this->data->setFileDir("javascript", $_path["project"]["script"]);
   $this->data->setFileExtensions("javascript", "js");
   $this->data->setFileDir("stylesheet", $_path["project"]["style"]);
   $this->data->setFileExtensions("stylesheet", "css");

   if(!$this->simpleFields())
   {
    $_PAGER_TEMPLATES_INIT=false;
    return false;
   }

   if(!$this->getMarkers())
   {
    $_PAGER_TEMPLATES_INIT=false;
    return false;
   }

   $pages=Plugin_Module_pager_categories::getPages();
   foreach($this->linkrels as $rel)
    $this->data->listOptions("link".$rel, $pages);

   $_PAGER_TEMPLATES_INIT=false;
   return true;
  }

  function copyPostGet(){return $this->getMarkers();}
  function editPostGet(){return $this->getMarkers();}
  function deletePostGet(){return $this->getMarkers();}

  function simpleFields()
  {
   global $_pager_simple;

   //hide "advanced" fields in simple mode
   if($_pager_simple)
   {
    $this->data->field["catnr"]["properties"].=";hidden";

    $this->data->field["markers"]["properties"].=";hidden";
    $this->data->field["doctype"]["properties"].=";hidden";
    $this->data->field["jasscript"]["properties"].=";hidden";

    $this->data->field["fixscript"]["properties"].=";hidden";
    $this->data->field["fixstyle"]["properties"].=";hidden";
    $this->data->field["fiximages"]["properties"].=";hidden";

    foreach($this->linkrels as $rel)
     $this->data->field["link".$rel]["properties"].=";hidden";
   }
   else
   {
    $this->data->resetField("catnr");

    $this->data->resetField("markers");
    $this->data->resetField("doctype");
    $this->data->resetField("jasscript");

    $this->data->resetField("fixscript");
    $this->data->resetField("fixstyle");
    $this->data->resetField("fiximages");

    foreach($this->linkrels as $rel)
     $this->data->resetField("link".$rel);
   }

   return true;
  }

  function getMarkers()
  {
   global $_path;

   if($this->data->action!="add"&&$this->data->action!="copy"&&$this->data->action!="edit"&&$this->data->action!="delete") //only when editting
    return true;

   if(!$this->data->getValue("template")) //no template selected
    return true;

   if($this->data->check("properties", "hidden", "markers")) //markers field not visible
    return true;

   //find markers in template
   $markers=Array();
   if($template=FS::readS($_path["project"]["templates"].$this->data->getValue("template")))
    foreach($this->data->listOptions["markers"] as $marker)
     if(strpos($template, "<!".$marker.">")||strpos($template, "#".$marker."#"))
      $markers[]=$marker;

   $this->data->setValue("markers", $markers);
   return true;
  }
 }
?>
