<?

 /*****************************************************************************
 * Filename: include/plugins/pager/pager.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."dt.php");
 require_once($_path["include"]."jas.php");
 require_once($_path["include"]."stp.php");
 require_once($_path["include"]."data.php");
 require_once($_path["include"]."cache.php");
 require_once($_path["include"]."layout.php");
 require_once($_path["format"]."template.php");

 class Plugin_pager extends Base_Plugin
 {
  function main()
  {
   global $_path, $_config, $_benchmark;

   if($_benchmark) $_benchmark->start("Page");

   if(!$this->config)
   {
    $this->e->add("No configuration loaded", "error", "Sorry this page could not be loaded");
    return true; //not fatal (output will render error page)
   }

   if($page=request("p")) //can be array or page nr/name
   {
    if(is_array($page)) //rewrite, use first argument
    {
     if(isset($page[0]))
      $page=$page[0];
     else
      $page=false;
    }
   }

   if(!$page)
   {
    if($this->config["startpagenr"])
     $page=$this->config["startpagenr"];
    else
     $page=$this->config["rootpagenr"];
   }


   if($_benchmark) $_benchmark->start("Get page info");

   //get page
   $d=Data::cache("pager_categories", "view", "all");
   $d->setShowFields("name", "text", "keywords", "urlname", "templatenr", "doctype", "javascriptglobal", "javascript", "stylesheet", "disabled", "updated", "symlink", "pagenr",
    "linkauthor", "linkcontact", "linkhelp", "linkindex", "linklicense", "linksearch", "linkfirst", "linkprev", "linknext", "linklast", "linkup");
   $d->setValue("show_total", 1);
   $d->clearValues();

   if(preg_match("/^[0-9]+$/", $page)) //page number
    $d->setValue("nr", $page);
   else //page name
    $d->setValue("urlname", $page);

   if(!$d->get(1))
    $this->e->add("Unable to get the page", "error", "Sorry this page could not be loaded");
   else if(!$d->next())
    $this->e->add("Page not found", "error", "Sorry this page could not be found");
   else if($d->v["disabled"])
    $this->e->add("Page not available", "error", "Sorry this page is not available at the moment");
   else
   {
    if($d->v["symlink"])
    {
     $d->setValue("show_total", 1);
     $d->clearValues();
     $d->setValue("nr", $d->v["pagenr"]);

     if(!$d->get(1))
      $this->e->add("Unable to get the page", "error", "Sorry this page could not be loaded");
     else if(!$d->next())
      $this->e->add("Page not found", "error", "Sorry this page could not be found");
     else if($d->v["disabled"])
      $this->e->add("Page not available", "error", "Sorry this page is not available at the moment");
    }

    if(!$d->v["templatenr"])
     $this->e->add("No template specified", "error", "Page should have a template specified to be displayed");
   }

   if($this->e->any("error"))
    return true; //not fatal (output will render error page)

   $this->lastUpdate=DT::dateConvertS($d->v["updated"], $d->field["updated"]["format"][$d->dataformat]["datetimeformat"]); //page last updated
   $this->pageID=$d->v["nr"]; //get page number;
   $this->pageName=$d->v["urlname"]; //get page name (will be id if not set)

   //get page markers
   $dm=Data::cache("pager_markers", "view", "all");
   $dm->setValue("templatenr", $d->v["templatenr"]["nr"]);
   $dm->setShowFields("nr", "marker", "clean");

   if(!$dm->get(1))
   {
    $this->e->add("Unable to get the page markers", "error", "Sorry this page could not be loaded");
    return true; //not fatal (output will render error page)
   }

   $markers=Array();
   $cleanMarkers=Array();
   while($dm->next())
   {
    $markers[$dm->v["nr"]]=$dm->v["marker"];

    if($dm->v["clean"])
     $cleanMarkers[]=$dm->v["marker"];
   }

   unset($dm);

   if(!$markers)
   {
    $this->e->add("No page markers found", "error", "Sorry this page could not be loaded");
    return true; //not fatal (output will render error page)
   }

   if($_benchmark) $_benchmark->stop("Get page info");



   if($_benchmark) $_benchmark->start("Setup template");

   $this->template=Array();
   $this->template["xml"]=false;
   $this->template["title"]=false;
   $this->template["description"]=false;
   $this->template["keywords"]=Array();
   $this->template["linkrel"]=Array();
   $this->template["jasscript"]=Array();
   $this->template["javascript"]=Array();
   $this->template["stylesheet"]=Array();
   $this->template["doctype"]=Array();

   $cache=new Cache("file");
   if($cache->e->any("fatal")) //caching prolly disabled
    $cache=false;
   else
   {
    $cacheKey=$this->config["plugin"];
    $cacheKey.="_template_".$d->v["templatenr"]["nr"];

    if($tmp=str_replace("/", "_", trim($_path["HTML"]["current"], "/"))) //add current HTML path
     $cacheKey.="_".$tmp;

    $cacheKey.=".html";

    //template record updated
    $cacheTime=DT::dateConvertS($d->v["templatenr"]["updated"], $d->reference["templatenr"]["data"]->field["updated"]["format"][$d->reference["templatenr"]["data"]->dataformat]["datetimeformat"]);

    if($this->lastUpdate<$cacheTime) //template updated after page entry?
     $this->lastUpdate=$cacheTime;

    //template updated
    $time=FS::getTimeS($_path["project"]["templates"].$d->v["templatenr"]["template"], "modified");

    if($time>$cacheTime)
     $cacheTime=$time;

    if(($time=$cache->time($cacheKey))!==false) //exists
    {
    if($this->lastUpdate<$time) //template file updated after page entry?
     $this->lastUpdate=$time;

     if($cacheTime==$time) //cache up-to-date
     {
      if(($xml=$cache->read($cacheKey))!==false) //no error, use the cache
      {
       $this->template["xml"]=new Format_Template(false);

       if(!$this->template["xml"]->parse($xml))
       {
        $this->e->add($this->template["xml"]->e);
        return true; //not fatal (output will render error page)
       }

       $this->template["xml"]->filename($cacheKey);
      }
      //else //should prolly remove it... then again... this is impossible
     }
     else //cache out of date
      $cache->remove($cacheKey); //remove the cache and re-cache after template is succesfully parsed
    }
    //else //doesn't exist yet, cache it at after template is succesfully parsed
   }

   if(!$this->template["xml"])
   {
    if(!($xml=FS::readS($_path["project"]["templates"].$d->v["templatenr"]["template"])))
    {
     $this->e->add("Unable to load template: \"".$_path["project"]["templates"].$d->v["templatenr"]["template"]."\"", "error");
     return true; //not fatal (output will render error page)
    }

    if($d->v["templatenr"]["fixscript"])
     $xml=preg_replace("!(<script.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $xml);

    if($d->v["templatenr"]["fixstyle"])
     $xml=preg_replace("!(<link.*css.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $xml);

    if($d->v["templatenr"]["fiximages"])
    {
     $xml=preg_replace("!(<link.*ico.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $xml);
     $xml=preg_replace("!(<img.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $xml);
    }

    $this->template["xml"]=new Format_Template(false);

    if(!$this->template["xml"]->parse($xml))
    {
     $this->e->add($this->template["xml"]->e);
     return true; //not fatal (output will render error page)
    }

    //add/update doctype
    list($version, $mode)=explode(" - ", $d->v["templatenr"]["doctype"]);

    if($this->template["xml"]->getKey("DOCTYPE")===false)
     $this->template["xml"]->addBefore("html", Layout::doctypeS($version, $mode));
    else
     $this->template["xml"]->setReplace("DOCTYPE", substr(Layout::doctypeS($version, $mode), 0, -1)); //stripping additional linebreak

    //add/update title
    if($this->template["xml"]->getKey("html/head/title")===false)
    {
     $this->template["xml"]->addInsideAtBegin("html/head", "<title>".$d->v["templatenr"]["title"]."</title>\n");
     $this->template["xml"]->reparse(true);
    }
    else
     $this->template["xml"]->setInside("html/head/title", $d->v["templatenr"]["title"]);

    //add JAVASCRIPT/JASSCRIPT marker
    if($this->template["xml"]->getKey("html/head/JAVASCRIPT")===false)
    {
     //find last script tag
     $tag=false;
     $this->template["xml"]->reset();
     while($this->template["xml"]->search("script"))
     {
      if(isset($this->template["xml"]->v["attributes"]["type"])&&$this->template["xml"]->v["attributes"]["type"]=="text/javascript")
       if($this->template["xml"]->getPath($this->template["xml"]->v["parent"])=="html[0]->head[0]") //must be in html header
        $tag=$this->template["xml"]->vc;
     }

     if($tag) //add after last stylesheet tag
      $this->template["xml"]->addAfter($tag, "\n<!JAVASCRIPT>");
     else
      $this->template["xml"]->addInsideAtEnd("html/head", "\n<!JAVASCRIPT>");
    }

    if($this->template["xml"]->getKey("html/head/JASSCRIPT")===false)
     $this->template["xml"]->addBefore("html/head/JAVASCRIPT", "<!JASSCRIPT>\n");

    //add STYLESHEET marker
    if($this->template["xml"]->getKey("html/head/STYLESHEET")===false)
    {
     //find last stylesheet tag
     $tag=false;
     $this->template["xml"]->reset();
     while($this->template["xml"]->search("link"))
     {
      if(isset($this->template["xml"]->v["attributes"]["rel"])&&$this->template["xml"]->v["attributes"]["rel"]=="stylesheet")
       if($this->template["xml"]->getPath($this->template["xml"]->v["parent"])=="html[0]->head[0]") //must be in html header
        $tag=$this->template["xml"]->vc;
     }

     if($tag) //add after last stylesheet tag
      $this->template["xml"]->addAfter($tag, "\n<!STYLESHEET>");
     else
      $this->template["xml"]->addInsideAtEnd("html/head", "\n<!STYLESHEET>");
    }

    //add template javascript
    foreach($d->v["templatenr"]["javascriptglobal"] as $file)
    {
     $tmp="  <script type=\"text/javascript\" src=\"".$_path["script"].$file."\"></script>\n";
     $this->template["xml"]->addBefore("html/head/JAVASCRIPT", $tmp);
    }
    foreach($d->v["templatenr"]["javascript"] as $file)
    {
     $tmp="  <script type=\"text/javascript\" src=\"".$_path["project"]["script"].$file."\"></script>\n";
     $this->template["xml"]->addBefore("html/head/JAVASCRIPT", $tmp);
    }

    //add template style sheets
    foreach($d->v["templatenr"]["stylesheet"] as $file)
    {
     $tmp="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$_path["project"]["style"].$file."\"/>\n";
     $this->template["xml"]->addBefore("html/head/STYLESHEET", $tmp);
    }

    //clean markers
    foreach($cleanMarkers as $marker)
     $this->template["xml"]->blank(html_entity_decode($marker), ENT_QUOTES|ENT_XHTML); //empty tag

    //cache template
    if(!$this->template["xml"]->e->any()&&$cache&&$cacheKey&&$cacheTime)
    {
     if(!$cache->save($cacheKey, $this->template["xml"]->output(), $cacheTime))
     {
      $this->e->add($cache->e);
      return true; //not fatal (output will render error page)
     }
    }
   }


   //page specific doctype
   if($d->v["doctype"]&&$d->v["doctype"]!="Use template") //custom doctype
   {
    list($version, $mode)=explode(" - ", $d->v["doctype"]);
    $this->setDoctype($version, $mode);
   }

   //page specific title
   if($d->v["templatenr"]["title"])
    $this->setTitle($d->v["templatenr"]["title"]." - ".$d->v["name"]);
   else
    $this->setTitle($d->v["name"]);

   //page description
   if($d->v["text"])
    $this->setDescription($d->v["text"]);

   //default keywords
   if($d->v["templatenr"]["keywords"])
    $this->addKeywords($d->v["templatenr"]["keywords"]);

   //page specific keywords (added before default keywords)
   if($d->v["keywords"])
    $this->addKeywords($d->v["keywords"]);

   //default link relations
   foreach($d->getReferenceData("templatenr")->moduleObject->linkrels as $rel)
    if($d->v["templatenr"]["link".$rel])
     $this->setLinkRelation($rel, $d->v["templatenr"]["link".$rel]);

   //page specific link relations
   foreach($d->moduleObject->linkrels as $rel)
    if($d->v["link".$rel])
     $this->setLinkRelation($rel, $d->v["link".$rel]);

   if($d->v["templatenr"]["jasscript"]) //(root) path dynamic so it can't be cached
    $this->jasScript(true);
   else
    $this->jasScript(false);

   //page specific javascript
   foreach($d->v["javascriptglobal"] as $file)
    $this->addJavaScript($_path["script"].$file);

   foreach($d->v["javascript"] as $file)
    $this->addJavaScript($_path["project"]["script"].$file);

   //page specific stylesheets
   foreach($d->v["stylesheet"] as $file)
    $this->addStyleSheet($_path["project"]["style"].$file);

   if($_benchmark) $_benchmark->stop("Setup template");



   if($_benchmark) $_benchmark->start("Main plugin logic");

   //page content, used later for rendering
   $contentCounter=0;
   $this->content=Array();

   $dc=Data::cache("pager_content", "view", "all");
   $dc->setShowFields("markernr", "module", "contentnr", "cache", "displayfor", "fixscript", "fixstyle", "fiximages", "updated");
   $dc->sortOrder("markernr", "ordernr");

   $dc->setValue("catnr", $this->config["rootpagenr"], 0);
   $dc->setValue("disabled", false, 0);
   $dc->setOperant("catnr", "OR");
   $dc->setValue("catnr", $this->pageID, 1);
   $dc->setValue("disabled", false, 1);

   if($cache)
   {
    $pluginCacheUpdated=false;
    if(!($pluginCache=$cache->read("pager_main_cache"))) //no cache yet
     $pluginCache=Array();
   }

   if(!$dc->get())
   {
    $this->e->add($dc->e);
    return true; //not fatal (output will render error page)
   }

   while($dc->next())
   {
    if(!isset($markers[$dc->v["markernr"]])) //not a marker for this template (must be gathered from rootpagenr)
     continue;

    //display for anonymous, user or root?
    if($dc->v["displayfor"]!="All")
    {
     //not logged in?
     if($dc->v["displayfor"]=="Anonymous"&&!$_config->userAnonymous())
      continue;

     //logged in?
     if($dc->v["displayfor"]=="Logged in"&&$_config->userAnonymous())
      continue;

     //are we root?
     if($dc->v["displayfor"]=="Root"&&!$_config->userRoot())
      continue;
    }

    if($_benchmark) $_benchmark->start($dc->v["module"]);

    $this->content[$contentCounter]["nr"]=$dc->v["nr"];
    $this->content[$contentCounter]["displayfor"]=$dc->v["displayfor"];
    $this->content[$contentCounter]["cache"]=Array("method" => $dc->v["cache"], "key" => false, "time" => false);

    $this->content[$contentCounter]["module"]=$dc->v["module"];
    $this->content[$contentCounter]["marker"]=html_entity_decode($markers[$dc->v["markernr"]], ENT_QUOTES|ENT_XHTML);

    $this->content[$contentCounter]["fixscript"]=$dc->v["fixscript"];
    $this->content[$contentCounter]["fixstyle"]=$dc->v["fixstyle"];
    $this->content[$contentCounter]["fiximages"]=$dc->v["fiximages"];


    $plugin=preg_replace("/_pluginConfigs$/", "", $dc->v["module"]);
    $this->content[$contentCounter]["plugin"]=new Plugin($plugin);

    //load plugin info+config
    $cacheKey=$plugin."|".$dc->v["contentnr"];
    if($cache&&isset($pluginCache[$cacheKey])&&$pluginCache[$cacheKey]["updated"]==$dc->v["updated"]) //cache exists and isn't updated (ignores actual pluginConfig last updated)
    {
     //restore info+config from cache
     $this->content[$contentCounter]["plugin"]->config=$pluginCache[$cacheKey]["config"];
     foreach($pluginCache[$cacheKey]["info"] as $field => $value)
      $this->content[$contentCounter]["plugin"]->$field=$value;
    }
    else
    {
     $this->content[$contentCounter]["plugin"]->info();
     $this->content[$contentCounter]["plugin"]->config($dc->v["contentnr"], true);

     //store info+config in cache
     if($cache)
     {
      $pluginCacheUpdated=true; //ensure plugin cache is stored
      $pluginCache[$cacheKey]=Array("updated" => $dc->v["updated"], "info" => Array(), "config" => $this->content[$contentCounter]["plugin"]->config);

      foreach($this->content[$contentCounter]["plugin"]->info->check("use", "show") as $field)
       $pluginCache[$cacheKey]["info"][$field]=$this->content[$contentCounter]["plugin"]->$field;
     }
    }

    $this->content[$contentCounter]["plugin"]->addVar("p", $this->pageName);
    $this->content[$contentCounter]["plugin"]->pager=$this; //set link to pager

    if(!$this->content[$contentCounter]["plugin"]->call("frontend", "main")) //execute main logic
     $this->content[$contentCounter]["plugin"]->failed=true; //don't render output

    if($_benchmark) $_benchmark->stop($this->content[$contentCounter]["module"]);

    $contentCounter++;
   }

   if($cache&&$pluginCacheUpdated) //update cache
   {
    if($cache->exists("pager_main_cache"))
    {
     if(!$cache->remove("pager_main_cache"))
     {
      $this->e->add($cache->e);
      return false;
     }
    }

    if(!$cache->save("pager_main_cache", $pluginCache))
    {
     $this->e->add($cache->e);
     return false;
    }
   }

   if($_benchmark) $_benchmark->stop("Main plugin logic");

   return true;
  }


  function output()
  {
   global $_path, $_config, $_session, $_benchmark;

   if($this->e->any("fatal"))
    return $this->error(); //render error page

   $cache=new Cache("file");
   if($cache->e->any("fatal")) //caching prolly disabled
    $cache=false;

   if($_config->get("stp"))
    $stp=new STP();
   else
    $stp=false;

   if($_benchmark) $_benchmark->start("Render template");

   //doctype
   if($this->template["doctype"]) //custom doctype
    $this->template["xml"]->setReplace("DOCTYPE", substr(Layout::doctypeS($this->template["doctype"]["version"], $this->template["doctype"]["mode"]), 0, -1)); //stripping additional linebreak

   //title
   if($this->template["title"]) //custom title
    $this->template["xml"]->setInside("html/head/title", $this->template["title"]);

   //keywords
   if($this->template["keywords"])
   {
    $tmp="\n  <meta name=\"keywords\" content=\"".implode(",", $this->template["keywords"])."\"/>";
    $this->template["xml"]->addAfter("html/head/title", $tmp);
   }

   //description
   if($this->template["description"])
   {
    $tmp="\n  <meta name=\"description\" content=\"".$this->template["description"]."\"/>";
    $this->template["xml"]->addAfter("html/head/title", "\n".$tmp);
   }

   //link relations
   if($this->template["linkrel"])
   {
    $tmp="";
    foreach($this->template["linkrel"] as $rel => $link)
     $tmp.="\n  <link rel=\"".$rel."\" href=\"".$link."\"/>";

    $this->template["xml"]->addAfter("html/head/title", "\n".$tmp);
   }

   //add JAS script
   if($this->template["jasscript"]) //add at begin of head
    $this->template["xml"]->setReplace("html/head/JASSCRIPT", JAS::JavaScriptS());

   //additional javascript
   foreach($this->template["javascript"] as $file)
   {
    $tmp="  <script type=\"text/javascript\" src=\"".$file."\"></script>\n";
    $this->template["xml"]->addBefore("html/head/JAVASCRIPT", $tmp);
   }

   $this->template["xml"]->delete("html/head/JAVASCRIPT");

   //additional stylesheet
   foreach($this->template["stylesheet"] as $file)
   {
    $tmp="  <link rel=\"stylesheet\" type=\"text/css\" href=\"".$file."\"/>\n";
    $this->template["xml"]->addBefore("html/head/STYLESHEET", $tmp);
   }

   $this->template["xml"]->delete("html/head/STYLESHEET");

   //set page last updated & generated
   $this->template["xml"]->exactPath=false;
   if($this->template["xml"]->getKey("LASTUPDATE")!==false)
    $this->template["xml"]->setReplace("LASTUPDATE", date($_config->get("datetimeformat"), $this->lastUpdate));
   if($this->template["xml"]->getKey("GENERATED")!==false)
    $this->template["xml"]->setReplace("GENERATED", date($_config->get("datetimeformat")));
   $this->template["xml"]->exactPath=true;


   //fix links, favicons, style sheets and javascript tags for rewrite prefix
   if($_config->get("prettyurls")&&$_path["HTML"]["prefix"])
   {
    //don't fix paths that start with a / or contain a : (http: / mailto:)
    $this->template["xml"]->output=preg_replace("!(<[^>]+href=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $this->template["xml"]->output);
    $this->template["xml"]->output=preg_replace("!(<[^>]+src=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $this->template["xml"]->output);
    $this->template["xml"]->reparse(true);
   }

   //add _benchmark hook
   if($_benchmark&&$this->plugin->config["benchmark"])
    if($this->template["xml"]->getKey("html/body")!==false)
     $this->template["xml"]->addInsideAtEnd("html/body", "#_benchmark#");

   //add _contentmarker hooks
   $l=sizeof($this->content);
   for($contentCounter=0;$contentCounter<$l;$contentCounter++)
    $this->template["xml"]->addInsideAtEnd($this->content[$contentCounter]["marker"], "#_contentmarker".$contentCounter."#");

   if($_benchmark) $_benchmark->stop("Render template");



   if($_benchmark) $_benchmark->start("Render content");

   #render all contents
   for($contentCounter=0;$contentCounter<$l;$contentCounter++)
   {
    if($_benchmark) $_benchmark->start($this->content[$contentCounter]["module"]);

    $content=false;
    $cacheKey=false;
    $cacheTime=false;
    if(isset($this->content[$contentCounter]["plugin"]->failed)) //main failed
    {
     //just show errors (no caching/output)
     if($this->content[$contentCounter]["plugin"]->config)
      $content="\n<a id=\"".str_replace(" ", "_", $this->content[$contentCounter]["plugin"]->config["key"])."\"></a>\n";

     $content.="<div class=\"".$this->content[$contentCounter]["plugin"]->name."\">\n";
     $content.=$this->content[$contentCounter]["plugin"]->e->show();
     $content.="\n</div>";
    }
    else if($cache&&$this->content[$contentCounter]["cache"]["method"]!="Disabled")
    {
     switch($this->content[$contentCounter]["cache"]["method"])
     {
      case "Generic":
      {
       $cacheKey=""; //use generic cache
       break;
      }
      case "Anonymous":
      {
       if($_config->userAnonymous())
        $cacheKey="anon_"; //use anonymous cache
       else
        $cacheKey=""; //use generic cache

       break;
      }
      case "Group":
      {
       $gids=$_config->get("groupnrs");
       sort($gids); //sort groups so users with same groups use same cache
       $cacheKey="gids_".implode("-", $gids)."_"; //use groups cache
       break;
      }
      case "User":
      {
       $cacheKey="uid_".$_config->get("usernr")."_"; //use user specific cache
       break;
      }
      default:
      {
       if($this->content[$contentCounter]["cache"]["method"]=="true"||$this->content[$contentCounter]["cache"]["method"]=="false") //old style caching
        $this->e->add("Unknown cache method: \"".$this->content[$contentCounter]["cache"]["method"]."\"", "error", "Please upgrade your database by running the following queries:
         ALTER TABLE `pager_content` CHANGE `cache` `cache` ENUM('true', 'false', 'Disabled', 'Generic', 'Anonymous', 'Group', 'User');
         UPDATE `pager_content` SET `cache`='Disabled' WHERE `cache`='false';
         UPDATE `pager_content` SET `cache`='Generic' WHERE `cache`='true';
         ALTER TABLE `pager_content` CHANGE `cache` `cache` ENUM('Disabled', 'Generic', 'Anonymous', 'Group', 'User');
        ");
       else
        $this->e->add("Unknown cache method: \"".$this->content[$contentCounter]["cache"]["method"]."\"", "error", "Please make sure all cache methods are properly implemented in the pager plugin");

       $cacheKey=""; //use generic cache
       break;
      }
     }

     $configUpdated=DT::dateConvertS($this->content[$contentCounter]["plugin"]->config["updated"], $_config->get("datetimeformat"));

     if($this->content[$contentCounter]["plugin"]->callExists("frontend", "cache")) //custom cache implementation
     {
      $cacheKey=$this->content[$contentCounter]["plugin"]->name."_".$cacheKey; //added plugin name prefix

      $contentUpdated=false;
      if($this->content[$contentCounter]["plugin"]->getClass("frontend")->cache($contentUpdated, $cacheKey))
      {
       if($contentUpdated>$configUpdated)
        $cacheTime=$contentUpdated;
       else
        $cacheTime=$configUpdated;
      }
      else //cache call failed, don't cache
       $cacheKey=false;
     }
     else //standard pager based cache implementation
     {
      $cacheTime=$configUpdated;
      $cacheKey=$this->config["plugin"]."_".$cacheKey."content_".$this->content[$contentCounter]["nr"].".html"; //generic content cache key name
     }

     if($this->lastUpdate<$cacheTime) //config/content updated after page entry?
      $this->lastUpdate=$cacheTime;

     if($cacheKey)
     {
      $this->content[$contentCounter]["cache"]["key"]=$cacheKey;
      $this->content[$contentCounter]["cache"]["time"]=$cacheTime;

      if(($time=$cache->time($cacheKey))!==false) //exists
      {
       if($this->lastUpdate<$time) //content template file updated after page entry?
        $this->lastUpdate=$time;

       if($cacheTime==$time) //cache up-to-date
        $content=$cache->read($cacheKey); //no error, use the cache
       else //cache out of date
        $cache->remove($cacheKey); //remove the cache and re-cache after template is succesfully parsed
      }
      //else //doesn't exist yet, cache it at after template is succesfully parsed
     }
    }

    if($content) //cached content
    {
     if($stp) //update STP to current
      $content=$stp->updateCurrent($content);
    }
    else //no cache/error getting cache
    {
     if($this->content[$contentCounter]["plugin"]->config)
      $content="\n<a id=\"".str_replace(" ", "_", $this->content[$contentCounter]["plugin"]->config["key"])."\"></a>\n";

     $content.="<div class=\"".$this->content[$contentCounter]["plugin"]->name."\">\n";

     //generate content
     if(($output=$this->content[$contentCounter]["plugin"]->call("frontend", "output"))===false) //failure generating content, won't be cached
     {
      $content.=$this->content[$contentCounter]["plugin"]->e->show();
      $content.="\n</div>";
     }
     else //successfully generate content
     {
      //change path
      if($this->content[$contentCounter]["fixscript"])
       $output=preg_replace("!(<script.* src=)(['\"])([^'\":]+)!i", "\\1\\2".$_path["project"]["templates"]."\\3", $output);

      if($this->content[$contentCounter]["fixstyle"])
       $output=preg_replace("!(<link.*css.* href=)(['\"])([^'\":]+)!i", "\\1\\2".$_path["project"]["templates"]."\\3", $output);

      if($this->content[$contentCounter]["fiximages"])
       $output=preg_replace("!(<img.* src=)(['\"])([^'\":]+)(['\"])!i", "\\1\\2".$_path["project"]["templates"]."\\3\\4", $output);

      $content.=$output;
      $content.="\n</div>";

      //save cached content
      if($cache&&$this->content[$contentCounter]["cache"]["key"])
       if(!$cache->save($this->content[$contentCounter]["cache"]["key"], $content, $this->content[$contentCounter]["cache"]["time"]))
        $content=$this->content[$contentCounter]["plugin"]->e->show().$content;
     }
    }

    if($_config->get("prettyurls")&&$_path["HTML"]["prefix"]) //fix links, favicons, style sheets and javascript tags for rewrite prefix
    {
     //don't fix paths that start with a / or contain a : (http: / mailto:)
     $content=preg_replace("!(<[^>]+href=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $content);
     $content=preg_replace("!(<[^>]+src=)(['\"])([^/][^'\"\:]+)(\?[^'\"]+)?(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"]."\\3\\4\\5", $content);
    }

    //replace _contentmarker hook with content
    $this->template["xml"]->output=str_replace("#_contentmarker".$contentCounter."#", $content, $this->template["xml"]->output);

    if($_benchmark) $_benchmark->stop($this->content[$contentCounter]["module"]);
   }

   //clean up unused _contentmarkers
   for($contentCounter=0;$contentCounter<$l;$contentCounter++)
    $this->template["xml"]->output=str_replace("#_contentmarker".$contentCounter."#", "", $this->template["xml"]->output);

   if($_benchmark) $_benchmark->stop("Render content");

   if($_benchmark) $_benchmark->stop("Page");

   if($_benchmark&&$this->plugin->config["benchmark"])
    $this->template["xml"]->output=str_replace("#_benchmark#", $_benchmark->show(), $this->template["xml"]->output);

   //we might need to reparse to catch errors... then agian, we probably already caught them all
   //$this->template["xml"]->reparse(true);

   if($this->template["xml"]->e->any())
   {
    $this->e->add($this->template["xml"]->e);
    return $this->error(); //render error page
   }

   return $this->template["xml"]->output();
  }



  function setDoctype($version, $mode)
  {
   $this->template["doctype"]["version"]=$version;
   $this->template["doctype"]["mode"]=$mode;
   return true;
  }
  function getDoctype()
  {
   return $this->template["doctype"]["version"].($this->template["doctype"]["mode"]?" - ".$this->template["doctype"]["mode"]:"");
  }


  function setTitle($title)
  {
   $this->template["title"]=$title;
   return true;
  }
  function getTitle()
  {
   return $this->template["title"];
  }


  function setDescription($description)
  {
   $this->template["description"]=$description;
   return true;
  }
  function getDescription()
  {
   return $this->template["description"];
  }

  function addKeywords($keywords)
  {
   if(!is_array($keywords))
   {
    if(strpos($keywords, ",")) //contains multiple keywords
     $keywords=preg_split("/\s*,\s*/", $keywords); //explode on ,
    else //single value
     $keywords=func_get_args(); //get all keywords
   }

   $this->template["keywords"]=array_merge($keywords, $this->template["keywords"]); //add latest keywords in front
   $this->template["keywords"]=array_unique($this->template["keywords"]);
   return true;
  }
  function getKeywords()
  {
   return $this->template["keywords"];
  }

  function setLinkRelation($rel, $link)
  {
   //Relations used by templates+categories module:
   //"author", "contact", "help", "index", "license", "search",
   //"first", "prev", "next", "last", "up"

   $this->template["linkrel"][$rel]=$link;
   return true;
  }
  function getLinkRelation($rel)
  {
   if(isset($this->template["linkrel"][$rel]))
    return $this->template["linkrel"][$rel];

   return false;
  }

  function jasScript($show)
  {
   $this->template["jasscript"]=($show?true:false);
   return true;
  }
  function getJasScript()
  {
   return $this->template["jasscript"];
  }

  function addJavaScript($file)
  {
   $this->template["javascript"][]=$file;
   return true;
  }
  function getJavaScript()
  {
   return $this->template["javascript"];
  }

  function addStyleSheet($file)
  {
   $this->template["stylesheet"][]=$file;
   return true;
  }
  function getStyleSheet()
  {
   return $this->template["stylesheet"];
  }



  function error()
  {
   global $_path, $_benchmark;

   if($_benchmark) $_benchmark->start("Setup error template");

   $this->plugin->info(); //load error
   $template=new Format_Template(true);

   if($this->config&&$this->config["errorTemplate"])
   {
    if(!file_exists($_path["project"]["templates"].$this->config["errorTemplate"]))
    {
     $template->read($this->plugin->path["templates"].$this->plugin->errorTemplate);

     $this->e->clear();
     $template->e->add("Error template: \"".$_path["project"]["templates"].$this->config["errorTemplate"]."\" not found", "system");
    }
    else
    {
     $template->read($_path["project"]["templates"].$this->config["errorTemplate"]);

     $template->output=preg_replace("!(<script.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->output);
     $template->output=preg_replace("!(<link.*css.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->output);
     $template->output=preg_replace("!(<link.*ico.* href=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->output);
     $template->output=preg_replace("!(<img.* src=)(['\"])([^'\"]+)(['\"])!i", "\\1\\2".$_path["HTML"]["prefix"].$_path["project"]["templates"]."\\3\\4", $template->output);
     $template->reparse(true);
    }
   }
   else if(isset($this->plugin->errorTemplate))
    $template->read($this->plugin->path["templates"].$this->plugin->errorTemplate);
   else
    $template->parse("<html><head><title>#error#</title></head><body><div>#error#</div></body></html>");

   if($template->getKey("error")===false)
    $template->e->add("Marker: \"error\" not found in error template: \"".$template->filename."\"", "system");

   if($template->e->any("fatal"))
    return $template->e->show();

   while($template->getKey("error")!==false)
    $template->setReplace("error", $this->e->l->t($this->e->getError(0)));

   if($template->getKey("extra")!==false)
    $template->setReplace("extra", $this->e->l->t($this->e->getExtra(0)));

   $output=$template->output();

   if($_benchmark) $_benchmark->stop("Setup error template");
   return $output;
  }

 }

?>
