<?

 /*****************************************************************************
 * Filename: include/plugins/pager/setup.php
 *
 * This file is part of the JAS project.
 * Copyright 2003-2016 by Matthijs Schuurman
 *
 * The JAS project is free software under the terms of the
 * GNU Lesser General Public License version 3 or any later version.
 *
 * For information about the license please refer to:
 * LICENSE.txt in the root of the project or http://jas.junerules.com/license/
 *****************************************************************************/


 require_once($_path["include"]."data.php");

 class Plugin_Setup_pager extends Base_Plugin_Setup
 {
  function init()
  {
   global $_path;

   //frontend files
   $this->mkdir($_path["project"]["templates"]);
   $this->mkdir($_path["project"]["style"]);
   $this->mkdir($_path["project"]["images"]);
   $this->mkdir($_path["project"]["script"]);

   $this->file("setup/index.php", $_path["project"]["root"]);

   $this->file("setup/index.html", $_path["project"]["templates"]);
   $this->file("setup/error.html", $_path["project"]["templates"]);
   $this->file("setup/navigation.html", $_path["project"]["templates"]);

   $this->file("setup/default.css", $_path["project"]["style"]);
   $this->file("setup/print.css", $_path["project"]["style"]);
   $this->file("setup/favicon.ico", $_path["project"]["images"]);
   $this->file("setup/logo.png", $_path["project"]["images"]);

   $this->modules();

   return true;
  }

  function postSetup()
  {
   global $_config;

   //get pager root
   $d=Data::cache("categories", "view", "all");
   $d->clearValues();

   $d->setShowFields("nr");
   $d->setValue("module", "pager_pluginConfigs");
   $d->setOperator("module", "like");

   if(!$d->get()||!$d->next()) //pager root category doesn't exist
   {
    //get root category
    $d->setAction("tree");
    $d->setExtra("down");
    $d->clearValues();

    $d->setShowFields("nr");

    $d->setValue("show_from", 0);
    $d->setValue("show_total", 1);

    $d->setValue("catnr", 0);
    $d->setValue("treedepth", 0);

    if(!$d->get()||!$d->next())
    {
     $this->e->add("Root category not found");
     $this->e->add($d->e);
     return false;
    }

    $rootcatnr=$d->v["nr"];

    $d->setAction("add");
    $d->setExtra("item");
    $d->clearValues();

    $d->setValue("name", "Website");
    $d->setValue("text", "Website root category");

    $d->setValue("catnr", $rootcatnr);
    $d->setValue("module", Array("pluginConfigs", "pager_pluginConfigs", "pager_categories"));
    $d->setValue("moduleLock", false);

    if(!$d->submit())
    {
     $this->e->add($d->e);
     return false;
    }

    $this->e->add("Pager root added successfully", "note");
    $pagerrootcatnr=$d->getValue("nr");
   }
   else
    $pagerrootcatnr=$d->v["nr"];


   //get pager template
   $d=Data::cache("pager_templates", "view", "all");
   $d->setShowFields("nr");
   $d->clearValues();

   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   $d->setValue("catnr", $pagerrootcatnr);
   $d->setValue("name", "Default");

   if(!$d->get()||!$d->next()) //pager default template doesn't exist
   {
    $d->setAction("add");
    $d->setExtra("item");
    $d->clearValues();

    $d->setValue("title", $_config->get("title"));
    $d->setValue("catnr", $pagerrootcatnr);

    if(!$d->submit())
    {
     $this->e->add($d->e);
     return false;
    }

    $this->e->add("Pager default template added successfully", "note");
    $pagertemplatenr=$d->getValue("nr");
   }
   else
    $pagertemplatenr=$d->v["nr"];


   //create default template markers
   $d=Data::cache("pager_markers", "view", "all");
   $d->setShowFields("nr");
   $d->clearValues();

   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   $d->setValue("catnr", $pagerrootcatnr);

   if(!$d->get()||!$d->next()) //pager template markers don't exist
   {
    $d->setAction("add");
    $d->setExtra("item");
    $d->clearValues();

    $c=0;
    $d->setValue("title", "Navigation", $c);
    $d->setValue("templatenr", $pagertemplatenr, $c);
    $d->setValue("marker", "html/body/div/div#header/div#navigation", $c);
    $d->setValue("placing", "header", $c);
    $d->setValue("catnr", $pagerrootcatnr, $c);

    $c++;
    $d->setValue("title", "Main", $c);
    $d->setValue("templatenr", $pagertemplatenr, $c);
    $d->setValue("marker", "html/body/div/div#body/div#main", $c);
    $d->setValue("placing", "body", $c);
    $d->setValue("catnr", $pagerrootcatnr, $c);

    $c++;
    $d->setValue("title", "Control", $c);
    $d->setValue("templatenr", $pagertemplatenr, $c);
    $d->setValue("marker", "html/body/div/div#body/div#control", $c);
    $d->setValue("placing", "body", $c);
    $d->setValue("small", true, $c);
    $d->setValue("catnr", $pagerrootcatnr, $c);

    $c++;
    $d->setValue("title", "Footer", $c);
    $d->setValue("templatenr", $pagertemplatenr, $c);
    $d->setValue("marker", "html/body/div/div#footer", $c);
    $d->setValue("placing", "footer", $c);
    $d->setValue("catnr", $pagerrootcatnr, $c);

    if(!$d->submit())
    {
     $this->e->add($d->e);
     return false;
    }

    $this->e->add("Pager default template markers added successfully", "note");
   }


   //get pager home
   $d=Data::cache("pager_categories", "view", "all");
   $d->setShowFields("nr");
   $d->clearValues();

   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   $d->setValue("catnr", $pagerrootcatnr);
   $d->setValue("module", "pager_content");
   $d->setOperator("module", "like");

   if(!$d->get()||!$d->next()) //pager home category doesn't exist
   {
    $d->setAction("add");
    $d->setExtra("item");
    $d->clearValues();

    $d->setValue("name", "Home");
    $d->setValue("text", "Home page");

    $d->setValue("templatenr", $pagertemplatenr);
    $d->setValue("catnr", $pagerrootcatnr);
    $d->setValue("module", Array("pluginConfigs", "pager_content"));
    $d->setValue("moduleLock", false);

    if(!$d->submit())
    {
     $this->e->add($d->e);
     return false;
    }

    $this->e->add("Pager home page added successfully", "note");
    $pagerhomecatnr=$d->getValue("nr");
   }
   else
    $pagerhomecatnr=$d->v["nr"];

   //create default pluginConfig
   $d=Data::cache("pager_pluginConfigs", "view", "all");
   $d->setShowFields("nr");
   $d->clearValues();

   $d->setValue("show_from", 0);
   $d->setValue("show_total", 1);

   $d->setValue("catnr", $pagerrootcatnr);
   $d->setValue("key", "default");

   if(!$d->get()||!$d->next()) //pager default pluginConfig doesn't exist
   {
    $d->setAction("add");
    $d->setExtra("item");
    $d->clearValues();

    $d->setValue("rootpagenr", $pagerrootcatnr);
    $d->setValue("startpagenr", $pagerhomecatnr);

    $d->setValue("key", "default");
    $d->setValue("catnr", $pagerrootcatnr);

    if(!$d->submit())
    {
     $this->e->add($d->e);
     return false;
    }

    $this->e->add("Pager default configuration added successfully", "note");
    $pagerconfignr=$d->getValue("nr");
   }
   else
    $pagerconfignr=$d->v["nr"];


   return true;
  }

 }
?>
